@@ -4,7 +4,7 @@ from contextlib import contextmanager
 from itertools import islice
 from typing import Optional
 
-from dbnd import dbnd_config, log_metric, log_metrics
+from dbnd import get_dbnd_project_config, log_metric, log_metrics
 from dbnd._core.constants import TaskRunState
 from dbnd._core.task_run.task_run import TaskRun
 from dbnd._core.task_run.task_run_error import TaskRunError
@@ -62,10 +62,23 @@ def is_monitor_dag(dag_id):
     return dag_id == MONITOR_DAG_NAME
 
 
+def is_subdag_eligible_for_tracking(dag_id, tracking_list):
+    for tracked_dag_id in tracking_list:
+        if dag_id.startswith(f"{tracked_dag_id}."):
+            return True
+
+    return False
+
+
 def is_dag_eligable_for_tracking(dag_id):
     # If return value is None, we track all dags. If it's empty list - we don't track anything.
     tracking_list = get_tracking_dag_ids_from_airflow_json()
-    return tracking_list is None or dag_id in tracking_list or is_monitor_dag(dag_id)
+    return (
+        tracking_list is None
+        or dag_id in tracking_list
+        or is_subdag_eligible_for_tracking(dag_id, tracking_list)
+        or is_monitor_dag(dag_id)
+    )
 
 
 def new_execute(context):
@@ -84,11 +97,11 @@ def new_execute(context):
         result = execute(copied_operator, context)
         return result
 
-    # Do not print console logs for monitor dag
-    if is_monitor_dag(context["task_instance"].dag_id):
-        dbnd_config.set_values({"core": {"tracker": ["api"]}}, "tracking")
-
     try:
+
+        # Set that we are in Airflow tracking mode
+        get_dbnd_project_config().set_is_airflow_runtime()
+
         task_context = extract_airflow_context(context)
         # start operator execute run with current airflow context
         task_run = dbnd_tracking_start(
@@ -102,17 +115,24 @@ def new_execute(context):
             exc_info=True,
         )
 
+    from airflow.exceptions import AirflowRescheduleException
+
     # running the operator's original execute function
     try:
         with af_tracking_context(task_run, context, copied_operator):
             execute = get_execute_function(copied_operator)
             result = execute(copied_operator, context)
 
+    # Check if this is sensor task that is retrying - normal behavior and not really an exception
+    except AirflowRescheduleException:
+        dbnd_tracking_stop(finalize_run=False)
+        raise
     # catch if the original execute failed
     except Exception as ex:
         if task_run:
             error = TaskRunError.build_from_ex(ex, task_run)
             task_run.set_task_run_state(state=TaskRunState.FAILED, error=error)
+
         dbnd_tracking_stop()
         raise
 
